/*
 * ioGame
 * Copyright (C) 2021 - 2023  渔民小镇 （262610965@qq.com、luoyizhu@gmail.com） . All Rights Reserved.
 * # iohao.com . 渔民小镇
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package com.iohao.mmo.equip.service;

import com.iohao.mmo.common.config.GameCode;
import com.iohao.mmo.equip.entity.BasicEquipProperty;
import com.iohao.mmo.equip.entity.Equip;
import com.iohao.mmo.equip.repository.EquipRepository;
import lombok.AllArgsConstructor;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.data.mongodb.core.query.Update;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Objects;
import java.util.Optional;

/**
 * @author 唐斌
 * @date 2023-07-30
 * @description: 装备属性实现类
 */
@Service
@AllArgsConstructor
public class EquipService {
    final MongoTemplate mongoTemplate;
    final EquipRepository equipRepository;

    public Equip ofEquip(String id) {

        return  mongoTemplate.findById(id, Equip.class);
    }

    public List<Equip> listByUser(long userId) {
        Query query = new Query();
        query.addCriteria(Criteria.where("userId").is(userId));

        return  mongoTemplate.find(query,Equip.class);
    }

    public void save(Equip equip) {
        mongoTemplate.save(equip);
    }

    /**
     * 分配装备属性点
     * @param equip
     */
    public void allotEquip(Equip equip){
        //判定总点数是否合法
        Equip oldEquip = mongoTemplate.findById(equip.getId(), Equip.class);

        GameCode.objNotFound.assertTrue(oldEquip!=null);

        BasicEquipProperty basicEquipProperty = equip.getBasicEquipProperty();
        int saveAttrTotal = basicEquipProperty.getConstitution()+basicEquipProperty.getMagicPower()+basicEquipProperty.getPower()+basicEquipProperty.getEndurance()+basicEquipProperty.getAgile();

        GameCode.allotNotEnough.assertTrue(saveAttrTotal<=oldEquip.getAttrTotal());

        // 查询条件，如果数据存在更新
        Query query = new Query();
        query.addCriteria(Criteria.where("id").is(equip.getId()));

        // 更新的字段
        Update update = new Update();
        //获取分配的字段

        update.set("undistributedAttr", oldEquip.getAttrTotal()-saveAttrTotal);
        update.set("basicEquipProperty", basicEquipProperty);
        mongoTemplate.upsert(query,update,Equip.class);

    }
}
